@@ -8,23 +8,10 @@ import os
 import logging
 import sys
 today = pd.to_datetime(date.today())
-
+from veetility import utility_functions
 #%%-----------------------------
-# Initialise Logger
-# ------------------------------
 
-logger = logging.getLogger('QAFunctions')
-if logger.hasHandlers():
-    logger.handlers = []
-if os.path.isdir('logs') == False:
-    os.mkdir('logs')
-logger.setLevel(logging.INFO)
-logger.addHandler(logging.StreamHandler(sys.stdout))
-formatter = logging.Formatter('%(levelname)s %(asctime)s - %(message)s')
-
-file_handler = logging.FileHandler(f'./logs/QAFunctions.log')
-file_handler.setFormatter(formatter)
-logger.addHandler(file_handler)
+qual_logger = utility_functions.Logger('Indeed','QualityAssessments')
 
 class QualityAssessments:
 
@@ -150,7 +137,7 @@ class QualityAssessments:
         if (missing_boosted) != 0:
             error_message = error_message + '  ' + \
                 f'There are {missing_boosted}({round(missing_boosted*100/paid_boosted_count,2)}%) posts mislabelled as Pure Organic\n'
-            logger.warning(error_message)
+            qual_logger.warning(error_message)
 
         # A post that is labelled as organic but has impressions over the impressions_threshold may actually be boosted 
         # but hasn't been identified as such
@@ -161,16 +148,16 @@ class QualityAssessments:
         if len(misslabelled_og_rows.index.values) > 0:
             error_message = error_message + '  ' + \
                 f'There are {len(misslabelled_og_rows.index.values)} Pure Organic posts with over {impressions_threshold} impressions or video views\n'
-            logger.warning(error_message)
+            qual_logger.warning(error_message)
             self.util.write_to_gsheet(gsheet_name,tab_name, misslabelled_og_rows)
         # return TT or IG values with empty message field
         # for the date function exlude the dates we paused for the queen
 
-        logger.warning(error_message)
+        qual_logger.warning(error_message)
         return error_message
     
     def comparison_with_previous_data(self,df,name_of_df,cols_to_check=['impressions','likes'],perc_increase_threshold=20,
-                                   perc_decrease_threshold=0.5, check_cols_set=True,raise_exceptions=False):
+                                   perc_decrease_threshold=0.5, check_cols_set=True,raise_exceptions=False, date_col=None):
         """ This function stores the high level sums for a datatable from the previous run of the script
         and if they have reduced or increased too sharply an error is raised.
         Args:
@@ -192,8 +179,13 @@ class QualityAssessments:
         error_message, error_occured = '', False
         new_dict = {}
         new_dict['datetime'] = str(datetime.now())
+        new_dict['num_rows'] = df.shape[0]
+        # if date_col != None:
+        #     new_dict['max_date'] = df[date_col].max()
+        #     new_dict['min_date'] = df[date_col].min()
+
         for col in cols_to_check:
-            new_dict[col] = df[col].sum()
+            new_dict[col] = int(df[col].sum())
 
         if check_cols_set == True:
             new_dict['Columns'] = df.columns.tolist()
@@ -202,13 +194,13 @@ class QualityAssessments:
             os.mkdir('Historic df Comparison (Do Not Delete)')
         if os.path.exists(f'Historic df Comparison (Do Not Delete)/{name_of_df}_previous_totals.json') ==False:
             self.util.write_json(new_dict,f'{name_of_df}_previous_totals',file_type='append',folder='Historic df Comparison (Do Not Delete)/')
-            logger.info(f"Creation of {name_of_df}_previous_totals")
-            logger.info(new_dict)
+            qual_logger.info(f"Creation of {name_of_df}_previous_totals")
+            qual_logger.info(new_dict)
             return
         else:
             #old_dict = self.util.unpickle_data(f'{name_of_df}_previous_totals',folder='Historic df Comparison (Do Not Delete)')
-            old_dict_file = self.util.read_json(f'{name_of_df}_previous_totals.json',folder='Historic df Comparison (Do Not Delete)',
-                                                    file_type='json')
+            old_dict_file = self.util.read_json(f'{name_of_df}_previous_totals',folder='Historic df Comparison (Do Not Delete)',
+                                                    file_type='append')
             old_dict = old_dict_file[-1] #Grab the lastest entry in the json file, descending date order
         
         for key, value in old_dict.items():
@@ -220,9 +212,16 @@ class QualityAssessments:
                     error_message = error_message + '  ' + f'The columns seems to have changed from last time,\n'\
                                             f' Columns that were added = {columns_added}\n' \
                                             f' Columns that were removed = {columns_removed}\n'
+            elif key == 'datetime':
+                continue
+            elif key == 'num_rows':
+                if new_dict[key] < value:
+                    error_occured = True
+                    error_message = error_message + '  ' + f'The number of rows has decreased from last time\n'\
+                                            f' Prev Value = {old_dict[key]} , New Value = {new_dict[key]}\n'
             elif new_dict[key] *(1+perc_decrease_threshold/100) < value:
                 error_occured = True
-                error_message = error_message + '  ' + f'The total of {key} seems to have decreased from last time\n'\
+                error_message = error_message + '  ' + f'The total of {key} seems to have decreased by more than {perc_decrease_threshold} from last time\n'\
                                         f' Prev Value = {old_dict[key]} , New Value = {new_dict[key]}\n'
             elif new_dict[key] > value*(1 +perc_increase_threshold/100):
                 error_occured = True
@@ -232,7 +231,7 @@ class QualityAssessments:
 
         if error_occured:
             error_message = f'Comparison with historic df {name_of_df}: ' + error_message +'\n'
-            logger.info('ERROR' + error_message) #if error messages has been added to then log it
+            qual_logger.info('ERROR - ' + error_message) #if error messages has been added to then log it
         if raise_exceptions and error_occured:
             raise Exception(error_message)
         #self.util.pickle_data(new_dict,f'{name_of_df}_previous_totals',folder='Historic df Comparison (Do Not Delete)/')
@@ -241,7 +240,7 @@ class QualityAssessments:
         
         return error_message
     
-    def duplicates_qa(self, df: pd.DataFrame, df_name: str,subset= None, drop_duplicates: bool = True):
+    def duplicates_qa(self, df: pd.DataFrame, df_name: str,subset= None, drop_duplicates: bool = True, cols_to_check = None):
         """Checks for duplicates and optionally drops duplicates in a dataframe.
         
         Args:
@@ -253,9 +252,11 @@ class QualityAssessments:
         Returns:
             df (pd.DataFrame): Returns original dataframe without duplicates if 'drop_duplicates' = True"""
 
+        if cols_to_check != None:
+            cols_to_check = ['']
         num_duplicates = df.duplicated(subset=subset).sum()
         if num_duplicates >= 0:
-            logger.warning(f'{num_duplicates} duplicates found in the {df_name}')
+            qual_logger.warning(f'{num_duplicates} duplicates found in the {df_name}. Subset = {subset}')
             if drop_duplicates:
                 df.drop_duplicates(subset=subset, inplace=True)
         return df
@@ -307,7 +308,7 @@ class QualityAssessments:
                             sheet_prefix=paid_or_organic)
             exception_occurred = True
 
-        logger.warning(error_message)
+        qual_logger.warning(error_message)
         if raise_exceptions and exception_occurred:
             raise Exception(error_message)
         
